from typing import Annotated
from contextlib import asynccontextmanager
import logging

from fastapi import FastAPI, Depends, BackgroundTasks, HTTPException, status
from fastapi.responses import FileResponse, HTMLResponse
from hatchet_sdk import EmptyModel
from jwt import decode as decode_jwt

from knoc.settings import settings
from knoc.models.foo import SimpleOutput, SimpleInput
from knoc.workflows.foo import foo_task, foo_workflow
from knoc.workflows.splynx import import_splynx_customers

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger("knoc")


@asynccontextmanager
async def lifespan(app: FastAPI):
    # Load the FastAPI app
    logger.debug("App is Starting Up!")
    yield
    logger.info(
        "App is Shutting Down Gracefully!",
        extra={"markup": True},
    )


app = FastAPI(lifespan=lifespan)


@app.get("/")
async def root():
    hatchet_console_url = decode_jwt(
        settings.hatchet_client.token, options={"verify_signature": False}
    )["server_url"]
    return HTMLResponse(
        f'<a href="{hatchet_console_url}">Hatchet console</a>'
        "<hr>"
        '<a href="/process/foo_task_bulk?messages=foo%20bar%20baz">foo workflow bulk</a>'
        "<hr>"
        '<a href="/process/foo_task?message=foo%20bar%20baz">foo task</a>'
        "<hr>"
        '<a href="/process/splynx-customer-import">Splynx customer import task</a>'
    )


@app.get("/process/splynx-customer-import")
async def process_splynx_customer_import():
    await import_splynx_customers.aio_run(EmptyModel())


@app.get("/process/foo_task_bulk")
async def process_foo_task_bulk(messages="Yo Foo") -> list[dict[str, SimpleOutput]]:
    results = await foo_workflow.aio_run_many(
        [
            # run each greeting as a task in parallel
            foo_workflow.create_bulk_run_item(
                input=SimpleInput(message=message),
            )
            for message in messages.split(" ")
        ]
    )
    return results


@app.get("/process/foo_task")
async def process_foo_task(message="Foo", no_wait=True) -> SimpleInput:
    if no_wait:
        ref = await foo_task.aio_run_no_wait(SimpleInput(message=message))
        result = await ref.aio_result()
    else:
        result = await foo_task.aio_run(SimpleInput(message="Hello, World!"))
    return result
